SystemOrganization addCategory: #'Soup-Core-Parser'!
SystemOrganization addCategory: #'Soup-Core'!
SystemOrganization addCategory: #'Soup-Core-TextConversion'!

TextConverter subclass: #ASCIITextConverter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Soup-Core-TextConversion'!

!ASCIITextConverter class methodsFor: 'encodings' stamp: 'StephaneDucasse 1/19/2011 22:09'!
encodingNames 
	^ #('ascii') copy
! !

!ASCIITextConverter methodsFor: 'friend' stamp: 'StephaneDucasse 1/19/2011 22:07'!
currentCharSize

	^ 1.
! !

!ASCIITextConverter methodsFor: 'conversion' stamp: 'StephaneDucasse 1/19/2011 22:07'!
nextFromStream: aStream
	| character |
	character := aStream basicNext.
	character charCode > 127 
		ifTrue: [self error: 'Invalid ASCII character ' , character asString].
	^ character
! !

!ASCIITextConverter methodsFor: 'conversion' stamp: 'StephaneDucasse 1/19/2011 22:07'!
nextPut: aCharacter toStream: aStream 
	aCharacter charCode > 127 
		ifTrue: [self error: 'Invalid ASCII character ' , aCharacter asString].
	aStream basicNextPut: (Character value: aCharacter charCode).
! !

!RxMatcher methodsFor: '*Soup-core-testing' stamp: 'za 2/15/2009 18:51'!
isMatcher
	^ true! !

!Object methodsFor: '*Soup-core-testing' stamp: 'za 12/22/2008 18:38'!
isMatcher
	^ false! !

Object subclass: #SoupDecoder
	instanceVariableNames: 'string encoding metaEncoding xmlEncoding sniffedXmlEncoding shouldConvertSmartQuotes convertSmartQuotesTo xmlForSmartQuotes htmlForSmartQuotes xmlData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Soup-Core-Parser'!

!SoupDecoder methodsFor: 'convert' stamp: 'za 12/27/2008 21:31'!
convertSmartQuote: aString
	^ convertSmartQuotesTo = #xml
		ifTrue: [self convertSmartQuoteToXml: aString]
		ifFalse: [self convertSmartQuoteToHtml: aString]
! !

!SoupDecoder methodsFor: 'convert' stamp: 'za 12/27/2008 21:11'!
convertSmartQuoteToHtml: aString
	^ '&{1};' format: {self htmlForSmartQuote: aString}
! !

!SoupDecoder methodsFor: 'convert' stamp: 'za 12/27/2008 21:10'!
convertSmartQuoteToXml: aString
	^ '&#x{1};' format: {self xmlForSmartQuote: aString}
! !

!SoupDecoder methodsFor: 'convert' stamp: 'za 12/27/2008 21:04'!
convertSmartQuotes: aString
	^ ('[{1}-{2}]' format: (#(128 159) collect: [:i | i asCharacter])) asRegex
		copy: aString
		translatingMatchesUsing: [
			:smartQuote |
			self convertSmartQuote: smartQuote
		]
! !

!SoupDecoder methodsFor: 'convert' stamp: 'za 12/27/2008 21:32'!
convertSmartQuotesToHtml
	convertSmartQuotesTo := #html
! !

!SoupDecoder methodsFor: 'convert' stamp: 'za 12/27/2008 21:32'!
convertSmartQuotesToXml
	convertSmartQuotesTo := #xml
! !

!SoupDecoder methodsFor: 'convert' stamp: 'StephaneDucasse 1/19/2011 22:12'!
decode

	self possibleEncodings
		do: [ :eachString |
				| decodedString |
				decodedString := self decodeFrom: eachString.
				decodedString ifNotNil: [^ decodedString]].
	^ string! !

!SoupDecoder methodsFor: 'convert' stamp: 'za 12/30/2008 04:06'!
decodeFrom: encodingName
	| converter |
	converter := TextConverter newForEncoding: encodingName.
	converter ifNil: [self error: 'No text converter for ' , encodingName].
	(self shouldConvertSmartQuotes: encodingName) 
		ifTrue: [xmlData := self convertSmartQuotes: xmlData].
	^ [xmlData convertFromWithConverter: converter]
		on: Error
		do: [:e | nil]! !

!SoupDecoder methodsFor: 'convert' stamp: 'za 12/30/2008 01:39'!
decodePythonString: aString
	^ String streamContents: [
		:aStream |
		(aString findTokens: '\x') do: [
			:eachString | 
			aStream nextPut: (Number 
				readFrom: eachString asUppercase 
				base: 16) asCharacter
		]
	]
	! !

!SoupDecoder methodsFor: 'accessing' stamp: 'za 12/30/2008 02:01'!
detectEncoding
	xmlData := string.
	[self sniffNonAsciiCompatibleEncoding.
	self sniffEncoding]
		on: Error do: [:e | ]
			! !

!SoupDecoder methodsFor: 'convert' stamp: 'za 12/27/2008 21:37'!
encodeAsUtf8: aString
! !

!SoupDecoder methodsFor: 'convert' stamp: 'za 12/30/2008 03:08'!
htmlForSmartQuote: aString
	^ htmlForSmartQuotes at: (aString at: 1) asInteger
! !

!SoupDecoder methodsFor: 'initialize' stamp: 'za 12/30/2008 02:21'!
initialize
	super initialize.
	self initializeXmlForSmartQuotes.
	self initializeHtmlForSmartQuotes.
	self convertSmartQuotesToXml.
	shouldConvertSmartQuotes := true! !

!SoupDecoder methodsFor: 'initialize' stamp: 'za 12/27/2008 21:34'!
initializeHtmlForSmartQuotes
	htmlForSmartQuotes := Dictionary newFrom: {
		128 -> 'euro'.
		129 -> ' '.
		130 -> 'sbquo'.
		131 -> 'fnof'.
		132 -> 'bdquo'.
		133 -> 'hellip'.
		134 -> 'dagger'.
		135 -> 'Dagger'.
		136 -> 'circ'.
		137 -> 'permil'.
		138 -> 'Scaron'.
		139 -> 'lsaquo'.
		140 -> 'OElig'.
		141 -> ' '.
		142 -> '#x17D'.
		143 -> ' '.
		144 -> ' '.
		145 -> 'lsquo'.
		146 -> 'rsquo'.
		147 -> 'ldquo'.
		148 -> 'rdquo'.
		149 -> 'bull'.
		150 -> 'ndash'.
		151 -> 'mdash'.
		152 -> 'tilde'.
		153 -> 'trade'.
		154 -> 'scaron'.
		155 -> 'rsaquo'.
		156 -> 'oelig'.
		157 -> ' '.
		158 -> '#x17E'.
		159 -> 'Yuml'.
	}.
! !

!SoupDecoder methodsFor: 'initialize' stamp: 'za 12/27/2008 21:34'!
initializeXmlForSmartQuotes
	xmlForSmartQuotes := Dictionary newFrom: {
		128 -> '20AC'.
		129 -> ' '.
		130 -> '201A'.
		131 -> '192'.
		132 -> '201E'.
		133 -> '2026'.
		134 -> '2020'.
		135 -> '2021'.
		136 -> '2C6'.
		137 -> '2030'.
		138 -> '160'.
		139 -> '2039'.
		140 -> '152'.
		141 -> ' '.
		142 -> '17D'.
		143 -> ' '.
		144 -> ' '.
		145 -> '2018'.
		146 -> '2019'.
		147 -> '201C'.
		148 -> '201D'.
		149 -> '2022'.
		150 -> '2013'.
		151 -> '2014'.
		152 -> '2DC'.
		153 -> '2122'.
		154 -> '161'.
		155 -> '203A'.
		156 -> '153'.
		157 -> ' '.
		158 -> '17E'.
		159 -> ''.
	}.
! !

!SoupDecoder methodsFor: 'convert' stamp: 'StephaneDucasse 1/19/2011 22:13'!
possibleEncodings
	| possibleEncodings |
	possibleEncodings := {
		encoding.
		metaEncoding.
		xmlEncoding.
		sniffedXmlEncoding.
	} asOrderedCollection reject: [:eachString | eachString isNil].
	(possibleEncodings includes: 'utf-8') 
		ifFalse: [possibleEncodings add: 'utf-8'].
	(possibleEncodings includes: 'cp-1252') 
		ifFalse: [possibleEncodings add: 'cp-1252'].
	^ possibleEncodings
	! !

!SoupDecoder methodsFor: 'convert' stamp: 'za 12/30/2008 02:10'!
reset
	xmlEncoding := nil.
	sniffedXmlEncoding := nil.
	string := nil.
	xmlData := nil.
	! !

!SoupDecoder methodsFor: 'convert' stamp: 'sd 1/20/2011 09:42'!
shouldConvertSmartQuotes: aString
	^ shouldConvertSmartQuotes and: [
		#(
			'cp-1252'
			'iso-8859-1'
			'iso-8859-2'
			'latin1'
		) includes: aString
	]
! !

!SoupDecoder methodsFor: 'accessing' stamp: 'za 12/30/2008 02:00'!
sniffEncoding
	| xmlEncodingMatcher |
	xmlEncodingMatcher := '^<\?.*encoding=[''"](.*?)[''"].*\?>' asRegex.
	(xmlEncodingMatcher matches: xmlData)
		ifTrue: [
  			xmlEncoding = (xmlEncodingMatcher subexpression: 1) asLowercase.
			(sniffedXmlEncoding notNil and: [
				#('iso-10646-ucs-2' 'ucs-2' 'csunicode'
				'iso-10646-ucs-4' 'ucs-4' 'csucs4'
				'utf-16' 'utf-32' 'utf_16' 'utf_32'
				'utf16' 'u16') includes: xmlEncoding
		]) ifTrue: [xmlEncoding := sniffedXmlEncoding]
	].
! !

!SoupDecoder methodsFor: 'accessing' stamp: 'sd 1/20/2011 09:31'!
sniffNonAsciiCompatibleEncoding

	(self stringStartsWith: '\x4c\x6f\xa7\x94')
		ifTrue: [
	  		sniffedXmlEncoding := nil.
			^ xmlData := self convertEbcdicToAscii: string.
		].
	(self stringStartsWith: '\x00\x3c\x00\x3f')
		ifTrue: [
			sniffedXmlEncoding := 'utf-16be'.
			^ xmlData := (string convertFromEncoding: sniffedXmlEncoding) 
				convertToEncoding: 'utf-8'
		].
	((string size >= 4) 
		and: [(self stringStartsWith: '\xfe\xff') 
			and: [(self stringStartsWith: '\xfe\xff\x00\x00') not]])
		ifTrue: [
			sniffedXmlEncoding := 'utf-16be'.
			^ xmlData := ((string copyFrom: 3 to: string size) 
				convertFromEncoding: sniffedXmlEncoding) 
					convertToEncoding: 'utf-8'
		].
	(self stringStartsWith: '\x3c\x00\x3f\x00')
		ifTrue: [
			sniffedXmlEncoding := 'utf-16le'.
			^ xmlData := (string convertFromEncoding: sniffedXmlEncoding) 
				convertToEncoding: 'utf-8'
		].
	((string size >= 4) 
		and: [(self stringStartsWith: '\xff\xfe') 
			and: [(self stringStartsWith: '\xff\xfe\x00\x00') not]])
		ifTrue: [
			sniffedXmlEncoding := 'utf-16le'.
			^ xmlData := ((string copyFrom: 3 to: string size) 
				convertFromEncoding: sniffedXmlEncoding) 
					convertToEncoding: 'utf-8'
		].
	(self stringStartsWith: '\x00\x00\x00\x3c')
		ifTrue: [
			sniffedXmlEncoding := 'utf-32be'.
			^ xmlData := (string convertFromEncoding: sniffedXmlEncoding) 
				convertToEncoding: 'utf-8'
		].
	(self stringStartsWith: '\x3c\x00\x00\x00')
		ifTrue: [
			sniffedXmlEncoding := 'utf-32le'.
			^ xmlData := (string convertFromEncoding: sniffedXmlEncoding) 
				convertToEncoding: 'utf-8'
		].
	(self stringStartsWith: '\x00\x00\xfe\xff')
		ifTrue: [
			sniffedXmlEncoding := 'utf-32be'.
			^ xmlData := ((string copyFrom: 5 to: string size) 
				convertFromEncoding: sniffedXmlEncoding) 
					convertToEncoding: 'utf-8'
		].
	(self stringStartsWith: '\xff\xfe\x00\x00')
		ifTrue: [
			sniffedXmlEncoding := 'utf-32le'.
			^ xmlData := ((string copyFrom: 5 to: string size) 
				convertFromEncoding: sniffedXmlEncoding) 
					convertToEncoding: 'utf-8'
		].
	(self stringStartsWith: '\xef\xbb\xbf')
		ifTrue: [
			sniffedXmlEncoding := 'utf-8'.
			^ xmlData := ((string copyFrom: 4 to: string size) 
				convertFromEncoding: sniffedXmlEncoding) 
					convertToEncoding: 'utf-8'
		].
	sniffedXmlEncoding := 'latin1'
! !

!SoupDecoder methodsFor: 'accessing' stamp: 'za 12/30/2008 01:18'!
string: anObject
	self reset.
	string := anObject.
	self detectEncoding.! !

!SoupDecoder methodsFor: 'accessing' stamp: 'za 3/9/2009 16:08'!
stringStartsWith: aString
	^ string beginsWith: (self decodePythonString: aString)
! !

!SoupDecoder methodsFor: 'convert' stamp: 'za 12/30/2008 03:04'!
xmlForSmartQuote: aString
	^ xmlForSmartQuotes at: (aString at: 1) asInteger
! !

Object subclass: #SoupElement
	instanceVariableNames: 'parent previous next previousSibling nextSibling'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Soup-Core'!

!SoupElement commentStamp: '<historical>' prior: 0!
SoupElement is the root of elements that are extracted during the parsing of an HTML document. Contrary to traditional HTML Parser, Soup produces a limited set of elements which approximate the HTML elements but supports querying and extracting information. !

!SoupElement class methodsFor: 'instance creation' stamp: 'za 12/10/2008 13:04'!
parent: anElement previous: anotherElement
	^ self new
		initializeWithParent: anElement previous: anotherElement;
		yourself! !

!SoupElement methodsFor: 'accessing' stamp: 'CamilloBruni 9/21/2011 15:09'!
@ aString
	^ self attributeAt: aString! !

!SoupElement methodsFor: 'accessing' stamp: 'sd 3/27/2011 19:13'!
attributeAt: aString 
	^ nil! !

!SoupElement methodsFor: 'accessing' stamp: 'sd 3/27/2011 19:15'!
attributeAt: aString ifAbsent: aBlock
	^ nil! !

!SoupElement methodsFor: 'accessing' stamp: 'sd 3/27/2011 19:13'!
attributeAt: aString put: anotherString

	! !

!SoupElement methodsFor: 'initializing' stamp: 'za 12/3/2008 18:43'!
beNextOf: anElement

	
	
! !

!SoupElement methodsFor: 'initializing' stamp: 'za 12/3/2008 18:43'!
beNextSiblingOf: anElement

	
	
! !

!SoupElement methodsFor: 'initializing' stamp: 'za 12/3/2008 18:36'!
beParentOf: anElement
! !

!SoupElement methodsFor: 'initializing' stamp: 'za 12/22/2008 19:46'!
bePreviousOf: anElement
	next := anElement! !

!SoupElement methodsFor: 'initializing' stamp: 'za 12/3/2008 18:45'!
bePreviousSiblingOf: anElement
	nextSibling := anElement
	
! !

!SoupElement methodsFor: 'finding strings' stamp: 'za 3/9/2009 16:45'!
createFindStringBlock: anObject
	(anObject isKindOf: Boolean) ifTrue: [^ [:aTag | anObject]].
	anObject isString ifTrue: [^ [:aString | aString contents = anObject]].
	anObject isMatcher ifTrue: [^ [:aString | anObject search: aString contents]].
	anObject isCollection ifTrue: [^ [:aString | anObject includes: aString contents]].
	^ anObject! !

!SoupElement methodsFor: 'finding tags' stamp: 'za 12/22/2008 22:40'!
createFindTagBlock: anObject
	(anObject isKindOf: Boolean) ifTrue: [^ [:aTag | anObject]].
	anObject isString ifTrue: [^ [:aTag | aTag name = anObject]].
	anObject isCollection ifTrue: [^ [:aTag | anObject includes: aTag name]].
	^ anObject
! !

!SoupElement methodsFor: 'finding strings' stamp: 'sd 12/13/2010 22:06'!
findAllStrings: aStringOrRegex
	"Returns all the strings equal to the argument. Note that the argument can be a regex to match several elements or a collection of strings"

	self subclassResponsibility! !

!SoupElement methodsFor: 'finding strings' stamp: 'CedrickBeler 8/30/2009 20:46'!
findAllStrings: anObject stream: aStream
	| findBlock |
	findBlock := self createFindStringBlock: anObject.
	self isString
			ifTrue: [(findBlock value: self) ifTrue: [aStream nextPut: self]]
			ifFalse:[ self childrenDo: [:ch | ch findAllStrings: anObject stream: aStream]].
! !

!SoupElement methodsFor: 'finding strings' stamp: 'za 12/23/2008 04:55'!
findChildString: anObject
	^ nil! !

!SoupElement methodsFor: 'finding tags' stamp: 'za 12/23/2008 04:55'!
findChildTag: anObject
	^ nil! !

!SoupElement methodsFor: 'finding strings' stamp: 'za 12/22/2008 22:41'!
findNextSiblingString: anObject
	| findBlock |
	nextSibling ifNil: [^ nil].
	findBlock := self createFindTagBlock: anObject.
	(nextSibling isString and: [findBlock value: nextSibling]) 
		ifTrue: [^ nextSibling].
	^ nextSibling findNextSiblingString: anObject! !

!SoupElement methodsFor: 'finding strings' stamp: 'za 12/22/2008 20:14'!
findNextSiblingStrings: anObject 
	| findBlock |
	findBlock := self createFindStringBlock: anObject.
	^ Array streamContents: [
		:aStream |
		self findNextSiblingStrings: findBlock stream: aStream
	]! !

!SoupElement methodsFor: 'finding strings' stamp: 'za 12/22/2008 20:18'!
findNextSiblingStrings: anObject stream: aStream
	| findBlock |
	nextSibling ifNil: [^ self].
	findBlock := self createFindStringBlock: anObject.
	(nextSibling isString and: [findBlock value: nextSibling]) 
		ifTrue: [aStream nextPut: nextSibling].
	nextSibling findNextSiblingStrings: anObject stream: aStream
! !

!SoupElement methodsFor: 'finding tags' stamp: 'sd 12/11/2010 16:41'!
findNextSiblingTag: anObject
	"Return the next sibling tag matching the argument. AnObject can be a Boolean, a block, a string, or a collection of strings"
	| findBlock |
	nextSibling ifNil: [^ nil].
	findBlock := self createFindTagBlock: anObject.
	(nextSibling isTag and: [findBlock value: nextSibling]) 
		ifTrue: [^ nextSibling].
	^ nextSibling findNextSiblingTag: anObject! !

!SoupElement methodsFor: 'finding tags' stamp: 'sd 12/11/2010 16:41'!
findNextSiblingTags: anObject 
	"Return the next sibling tags matching the argument. AnObject can be a Boolean, a block, a string, or a collection of strings"
	| findBlock |
	findBlock := self createFindTagBlock: anObject.
	^ Array streamContents: [ :aStream |
			self findNextSiblingTags: findBlock stream: aStream ]! !

!SoupElement methodsFor: 'finding tags implementation' stamp: 'za 12/22/2008 22:42'!
findNextSiblingTags: anObject stream: aStream
	| findBlock |
	nextSibling ifNil: [^ self].
	findBlock := self createFindTagBlock: anObject.
	(nextSibling isTag and: [findBlock value: nextSibling]) 
		ifTrue: [aStream nextPut: nextSibling].
	nextSibling findNextSiblingTags: anObject stream: aStream
! !

!SoupElement methodsFor: 'finding strings' stamp: 'za 12/22/2008 22:42'!
findNextString: anObject
	| findBlock |
	next ifNil: [^ nil].
	findBlock := self createFindTagBlock: anObject.
	(next isString and: [findBlock value: next]) ifTrue: [^ next].
	^ next findNextString: anObject! !

!SoupElement methodsFor: 'finding strings' stamp: 'za 12/22/2008 19:52'!
findNextStrings: anObject 
	| findBlock |
	findBlock := self createFindStringBlock: anObject.
	^ Array streamContents: [
		:aStream |
		self findNextStrings: findBlock stream: aStream
	]! !

!SoupElement methodsFor: 'finding strings' stamp: 'za 12/22/2008 20:19'!
findNextStrings: anObject stream: aStream
	| findBlock |
	next ifNil: [^ self].
	findBlock := self createFindStringBlock: anObject.
	(next isString and: [findBlock value: next]) ifTrue: [aStream nextPut: next].
	next findNextStrings: anObject stream: aStream
! !

!SoupElement methodsFor: 'finding tags' stamp: 'sd 12/11/2010 16:41'!
findNextTag: anObject
	"Return the next tag matching the argument. AnObject can be a Boolean, a block, a string, or a collection of strings"
	| findBlock |
	next ifNil: [^ nil].
	findBlock := self createFindTagBlock: anObject.
	(next isTag and: [findBlock value: next]) ifTrue: [^ next].
	^ next findNextTag: anObject! !

!SoupElement methodsFor: 'finding tags' stamp: 'sd 12/11/2010 16:35'!
findNextTags: anObject 
	| findBlock |
	findBlock := self createFindTagBlock: anObject.
	^ Array streamContents: [ :aStream |
			self findNextTags: findBlock stream: aStream ]! !

!SoupElement methodsFor: 'finding tags implementation' stamp: 'za 12/22/2008 22:45'!
findNextTags: anObject stream: aStream
	| findBlock |
	next ifNil: [^ self].
	findBlock := self createFindTagBlock: anObject.
	(next isTag and: [findBlock value: next]) ifTrue: [aStream nextPut: next].
	next findNextTags: anObject stream: aStream
! !

!SoupElement methodsFor: 'finding strings' stamp: 'za 12/22/2008 22:43'!
findPreviousSiblingString: anObject
	| findBlock |
	previousSibling ifNil: [^ nil].
	findBlock := self createFindTagBlock: anObject.
	(previousSibling isString and: [findBlock value: previousSibling]) 
		ifTrue: [^ previousSibling].
	^ previousSibling findPreviousSiblingString: anObject! !

!SoupElement methodsFor: 'finding strings' stamp: 'za 12/22/2008 20:20'!
findPreviousSiblingStrings: anObject 
	| findBlock |
	findBlock := self createFindStringBlock: anObject.
	^ Array streamContents: [
		:aStream |
		self findPreviousSiblingStrings: findBlock stream: aStream
	]! !

!SoupElement methodsFor: 'finding strings' stamp: 'za 12/22/2008 20:20'!
findPreviousSiblingStrings: anObject stream: aStream
	| findBlock |
	previousSibling ifNil: [^ self].
	findBlock := self createFindStringBlock: anObject.
	(previousSibling isString and: [findBlock value: previousSibling]) 
		ifTrue: [aStream nextPut: previousSibling].
	previousSibling findPreviousSiblingStrings: anObject stream: aStream
! !

!SoupElement methodsFor: 'finding tags' stamp: 'sd 12/11/2010 16:52'!
findPreviousSiblingTag: anObject
	"Return the previous sibling tag matching the argument. AnObject can be a Boolean, a block, a string, or a collection of strings"
	| findBlock |
	previousSibling ifNil: [^ nil].
	findBlock := self createFindTagBlock: anObject.
	(findBlock value: previousSibling) ifTrue: [^ previousSibling].
	^ previousSibling findPreviousSiblingTag: anObject! !

!SoupElement methodsFor: 'finding tags' stamp: 'sd 12/11/2010 16:51'!
findPreviousSiblingTags: anObject 
	"Return the previous sibling tags matching the argument. AnObject can be a Boolean, a block, a string, or a collection of strings"
	| findBlock |
	findBlock := self createFindTagBlock: anObject.
	^ Array streamContents: [ :aStream | 
			self findPreviousSiblingTags: findBlock stream: aStream ]! !

!SoupElement methodsFor: 'finding tags implementation' stamp: 'za 12/22/2008 22:43'!
findPreviousSiblingTags: anObject stream: aStream
	| findBlock |
	previousSibling ifNil: [^ self].
	findBlock := self createFindTagBlock: anObject.
	(previousSibling isTag and: [findBlock value: previousSibling]) 
		ifTrue: [aStream nextPut: previousSibling].
	previousSibling findPreviousSiblingTags: anObject stream: aStream
! !

!SoupElement methodsFor: 'finding strings' stamp: 'za 12/22/2008 22:43'!
findPreviousString: anObject
	| findBlock |
	previous ifNil: [^ nil].
	findBlock := self createFindTagBlock: anObject.
	(previous isString and: [findBlock value: previous]) ifTrue: [^ previous].
	^ previous findNextString: anObject! !

!SoupElement methodsFor: 'finding strings' stamp: 'za 12/22/2008 20:21'!
findPreviousStrings: anObject 
	| findBlock |
	findBlock := self createFindStringBlock: anObject.
	^ Array streamContents: [
		:aStream |
		self findPreviousStrings: findBlock stream: aStream
	]! !

!SoupElement methodsFor: 'finding strings' stamp: 'za 12/22/2008 22:35'!
findPreviousStrings: anObject stream: aStream
	| findBlock |
	previous ifNil: [^ self].
	findBlock := self createFindStringBlock: anObject.
	(previous isString and: [findBlock value: previous]) ifTrue: [aStream nextPut: previous].
	previous findPreviousStrings: anObject stream: aStream
! !

!SoupElement methodsFor: 'finding tags' stamp: 'sd 12/11/2010 16:52'!
findPreviousTag: anObject
	"Return the next tag matching the argument. AnObject can be a Boolean, a block, a string, or a collection of strings"
	| findBlock |
	previous ifNil: [^ nil].
	findBlock := self createFindTagBlock: anObject.
	(findBlock value: previous) ifTrue: [^ previous].
	^ previous findPreviousTag: anObject! !

!SoupElement methodsFor: 'finding tags' stamp: 'sd 12/11/2010 16:52'!
findPreviousTags: anObject 
	"Return the next tags matching the argument. AnObject can be a Boolean, a block, a string, or a collection of strings"
	| findBlock |
	findBlock := self createFindTagBlock: anObject.
	^ Array streamContents: [ :aStream |
			self findPreviousTags: findBlock stream: aStream ]! !

!SoupElement methodsFor: 'finding tags implementation' stamp: 'za 12/22/2008 22:41'!
findPreviousTags: anObject stream: aStream
	| findBlock |
	previous ifNil: [^ self].
	findBlock := self createFindTagBlock: anObject.
	(previous isTag and: [findBlock value: previous]) ifTrue: [aStream nextPut: previous].
	previous findPreviousTags: anObject stream: aStream
! !

!SoupElement methodsFor: 'testing' stamp: 'CamilloBruni 9/21/2011 15:09'!
hasAttribute: aString
	^ (self attributeAt: aString) isNil not! !

!SoupElement methodsFor: 'accessing' stamp: 'sd 3/27/2011 19:15'!
hasAttributes
	^ false! !

!SoupElement methodsFor: 'testing' stamp: 'za 12/22/2008 19:29'!
ifString: aBlock! !

!SoupElement methodsFor: 'testing' stamp: 'sd 12/13/2010 21:28'!
ifTag: aBlock
	"When the receiver is a tag execute the argument aBlock. The result of the block execution is discarded"! !

!SoupElement methodsFor: 'initializing' stamp: 'za 12/3/2008 18:37'!
initializeWithParent: anElement previous: anotherElement
	self parent: anElement.
	self previous: anotherElement.! !

!SoupElement methodsFor: 'testing' stamp: 'za 12/25/2008 07:01'!
isCData
	^ false! !

!SoupElement methodsFor: 'testing' stamp: 'za 12/25/2008 07:14'!
isComment 
	^ false! !

!SoupElement methodsFor: 'testing' stamp: 'za 12/25/2008 06:32'!
isDeclaration
	^ false! !

!SoupElement methodsFor: 'testing' stamp: 'za 12/25/2008 07:18'!
isProcessingInstruction
	^ false! !

!SoupElement methodsFor: 'testing' stamp: 'za 12/22/2008 19:18'!
isSoup
	^ false! !

!SoupElement methodsFor: 'testing' stamp: 'za 12/22/2008 19:18'!
isString
	^ false! !

!SoupElement methodsFor: 'testing' stamp: 'za 12/22/2008 19:18'!
isTag
	^ false! !

!SoupElement methodsFor: 'accessing' stamp: 'za 12/20/2008 10:33'!
name
	^ nil! !

!SoupElement methodsFor: 'accessing' stamp: 'za 12/22/2008 17:48'!
next
	^ next! !

!SoupElement methodsFor: 'accessing' stamp: 'za 12/3/2008 18:45'!
next: anElement
	anElement beNextOf: self.
	next := anElement

! !

!SoupElement methodsFor: 'accessing' stamp: 'za 12/22/2008 17:47'!
nextSibling
	^ nextSibling! !

!SoupElement methodsFor: 'accessing' stamp: 'za 12/3/2008 18:42'!
nextSibling: anElement
	anElement beNextSiblingOf: self.
	nextSibling := anElement.
	
	
! !

!SoupElement methodsFor: 'accessing' stamp: 'za 12/22/2008 18:43'!
parent
	^ parent! !

!SoupElement methodsFor: 'accessing' stamp: 'za 12/3/2008 18:45'!
parent: anElement
	anElement beParentOf: self.
	parent := anElement

! !

!SoupElement methodsFor: 'accessing' stamp: 'za 12/22/2008 17:48'!
previous
	^ previous! !

!SoupElement methodsFor: 'accessing' stamp: 'test 12/22/2008 20:47'!
previous: anElement
	anElement ifNotNil: [anElement bePreviousOf: self].
	previous := anElement

! !

!SoupElement methodsFor: 'accessing' stamp: 'za 12/22/2008 17:47'!
previousSibling
	^ previousSibling! !

!SoupElement methodsFor: 'accessing' stamp: 'za 12/3/2008 18:41'!
previousSibling: anElement
	anElement bePreviousSiblingOf: self.
	previousSibling := anElement.
! !

!SoupElement methodsFor: 'printing' stamp: 'za 12/22/2008 19:18'!
printOn: aStream 
	self subclassResponsibility
! !

SoupElement subclass: #SoupString
	instanceVariableNames: 'contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Soup-Core'!

SoupString subclass: #SoupCData
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Soup-Core'!

!SoupCData methodsFor: 'testing' stamp: 'za 12/25/2008 07:01'!
isCData
	^ true! !

!SoupCData methodsFor: 'printing' stamp: 'za 12/25/2008 07:13'!
printOn: aStream
	aStream 
		nextPutAll: '<!![CDATA[';
		nextPutAll: contents;
		nextPutAll: ']]>'! !

SoupString subclass: #SoupComment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Soup-Core'!

!SoupComment methodsFor: 'testing' stamp: 'za 12/25/2008 07:14'!
isComment
	^ true! !

!SoupComment methodsFor: 'printing' stamp: 'sd 1/7/2011 15:06'!
printOn: aStream
	aStream 
		nextPutAll: '<!!--';
		nextPutAll: contents;
		nextPutAll: '-->'! !

SoupString subclass: #SoupDeclaration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Soup-Core'!

!SoupDeclaration methodsFor: 'testing' stamp: 'za 12/25/2008 06:32'!
isDeclaration
	^ true! !

!SoupDeclaration methodsFor: 'printing' stamp: 'za 12/25/2008 06:42'!
printOn: aStream
	aStream 
		nextPutAll: '<!!';
		nextPutAll: contents;
		nextPutAll: '>'! !

SoupString subclass: #SoupProcessingInstruction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Soup-Core'!

!SoupProcessingInstruction methodsFor: 'testing' stamp: 'za 12/25/2008 07:19'!
isProcessingInstruction
	^ true! !

!SoupProcessingInstruction methodsFor: 'printing' stamp: 'za 12/25/2008 07:19'!
printOn: aStream
	aStream 
		nextPutAll: '<?';
		nextPutAll: contents;
		nextPutAll: '>'! !

!SoupString class methodsFor: 'instance creation' stamp: 'za 12/22/2008 22:44'!
parent: anElement previous: anotherElement contents: aString
	^ (super parent: anElement previous: anotherElement)
		contents: aString;
		yourself! !

!SoupString methodsFor: 'accessing' stamp: 'za 12/22/2008 22:47'!
contents
	^ contents! !

!SoupString methodsFor: 'accessing' stamp: 'za 12/22/2008 22:39'!
contents: anObject
	contents := anObject! !

!SoupString methodsFor: 'finding strings' stamp: 'sd 12/11/2010 15:54'!
findAllStrings: aStringOrRegex
	"There is no string inside a string"
	^ #()! !

!SoupString methodsFor: 'finding tags' stamp: 'za 12/22/2008 22:37'!
findAllTags: anObject
	^ #()! !

!SoupString methodsFor: 'finding strings' stamp: 'za 12/22/2008 22:37'!
findString: anObject
	^ nil! !

!SoupString methodsFor: 'finding tags' stamp: 'za 12/22/2008 22:37'!
findTag: anObject
	^ nil! !

!SoupString methodsFor: 'testing' stamp: 'za 12/22/2008 19:29'!
ifString: aBlock
	^ aBlock value.! !

!SoupString methodsFor: 'testing' stamp: 'za 12/9/2008 14:52'!
isString
	^ true! !

!SoupString methodsFor: 'printing' stamp: 'za 12/22/2008 22:39'!
printOn: aStream
	aStream nextPutAll: contents.
! !

!SoupString methodsFor: 'accessing' stamp: 'za 2/5/2009 03:01'!
readStream
	^ contents readStream! !

!SoupString methodsFor: 'accessing' stamp: 'za 12/25/2008 08:13'!
text
	^ contents! !

SoupElement subclass: #SoupTag
	instanceVariableNames: 'name attributes children isSelfClosing'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Soup-Core'!

SoupTag subclass: #Soup
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Soup-Core'!

!Soup commentStamp: '<historical>' prior: 0!
Soup is the main public class of the package. It is the class to interact with as shown by the tests.

The best way to get started is to get a soup of html elements using fromString: as follows:

	Soup fromString: 'http://www.bedetheque.com/album-105582-BD-Atomium-Express.html' asUrl retrieveContents contentStream


!

!Soup class methodsFor: 'licensing' stamp: 'za 12/30/2008 04:46'!
License
" Licensed like BeautifulSoup (see Soup OriginalLicense) "
	self error: 'comment only'! !

!Soup class methodsFor: 'licensing' stamp: 'za 12/30/2008 04:46'!
OriginalLicense
"Copyright (c) 2004-2007, Leonard Richardson

All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above
    copyright notice, this list of conditions and the following
    disclaimer in the documentation and/or other materials provided
    with the distribution.

  * Neither the name of the the Beautiful Soup Consortium and All
    Night Kosher Bakery nor the names of its contributors may be
    used to endorse or promote products derived from this software
    without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
""AS IS"" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE, DAMMIT.

"
	self error: 'comment only'! !

!Soup class methodsFor: 'defaults' stamp: 'za 12/26/2008 22:29'!
defaultName
	^ '[document]'! !

!Soup class methodsFor: 'defaults' stamp: 'za 12/20/2008 10:46'!
defaultParserParameters
	^ SoupParserParameters forHtml! !

!Soup class methodsFor: 'instance creation' stamp: 'za 12/20/2008 10:45'!
fromString: aString
	^ (SoupParser parameters: self defaultParserParameters) 
		parseString: aString! !

!Soup class methodsFor: 'instance creation' stamp: 'za 2/11/2009 02:39'!
fromUrl: aString
	Transcript show: aString; cr; flush.
	^ self fromString: (HTTPClient httpGet: aString) contents
! !

!Soup class methodsFor: 'instance creation' stamp: 'za 12/26/2008 22:29'!
new
	^ super new
		name: self defaultName;
		yourself! !

!Soup methodsFor: 'accessing' stamp: 'za 12/20/2008 11:00'!
addChild: aChild
	^ children add: aChild! !

!Soup methodsFor: 'error handling' stamp: 'za 12/21/2008 13:58'!
doesNotUnderstand: aMessage
	" Treat only unary messages as short hand child access. "
	(aMessage arguments size > 0)
		ifTrue: [^ super doesNotUnderstand: aMessage].
	self 
		findTag: aMessage selector
		ifPresent: [:aTag | ^ aTag].
	^ nil! !

!Soup methodsFor: 'initialization' stamp: 'za 12/23/2008 04:42'!
initialize
	super initialize.
	self name = '[document]'! !

!Soup methodsFor: 'testing' stamp: 'za 12/20/2008 10:59'!
isSoup
	^ true! !

!Soup methodsFor: 'printing' stamp: 'za 12/22/2008 21:59'!
printOn: aStream
	self childrenDo: [
		:eachElement |
		eachElement printOn: aStream.
	].
! !

!SoupTag class methodsFor: 'instance creation' stamp: 'za 12/22/2008 17:56'!
named: aString
	^ self new 
		name: aString;
		yourself! !

!SoupTag class methodsFor: 'instance creation' stamp: 'za 12/10/2008 16:24'!
named: aString parent: anElement previous: anotherElement
	^ (self parent: anElement previous: anotherElement)
		name: aString;
		yourself! !

!SoupTag methodsFor: 'finding tags' stamp: 'CamilloBruni 9/21/2011 17:17'!
/ anObject
	^ SoupXPath withAll: (self findAllChildTags: anObject)! !

!SoupTag methodsFor: 'finding tags' stamp: 'CamilloBruni 9/21/2011 17:17'!
// anObject
	^ SoupXPath withAll: (self findAllTags: anObject)! !

!SoupTag methodsFor: 'adding' stamp: 'za 12/9/2008 14:14'!
addChild: aChild
	^ children add: aChild! !

!SoupTag methodsFor: 'converting' stamp: 'CamilloBruni 9/21/2011 17:50'!
asXPath
	^ SoupXPath with: self! !

!SoupTag methodsFor: 'accessing' stamp: 'za 12/10/2008 16:55'!
attributeAt: aString 
	^ attributes at: aString ifAbsent: [nil]! !

!SoupTag methodsFor: 'accessing' stamp: 'sd 3/27/2011 19:14'!
attributeAt: aString ifAbsent: aBlock
	^ attributes at: aString ifAbsent: aBlock! !

!SoupTag methodsFor: 'accessing' stamp: 'za 12/7/2008 05:52'!
attributeAt: aString put: anotherString
	^ attributes at: aString put: anotherString! !

!SoupTag methodsFor: 'initializing' stamp: 'za 12/11/2008 00:34'!
beParentOf: anElement
	children ifNotEmpty: [anElement previousSibling: children last].
	children add: anElement.
	! !

!SoupTag methodsFor: 'accessing' stamp: 'za 12/10/2008 16:24'!
beSelfClosing
	isSelfClosing := true! !

!SoupTag methodsFor: 'accessing' stamp: 'sd 12/5/2010 22:50'!
childAt: anInteger
	"Return the anInteger th child of the receiver"
	^ children at: anInteger! !

!SoupTag methodsFor: 'enumerating' stamp: 'sd 12/13/2010 21:51'!
childStringsDo: aBlock
	"Apply the argument, an Unary block, on the direct children that are plain strings. Strings decorated (e.g., <p>, <b>...) are not iterated"
	
	self childrenDo: 
		[ :anElement |
			anElement ifString: [aBlock value: anElement]]! !

!SoupTag methodsFor: 'enumerating' stamp: 'sd 12/13/2010 21:51'!
childTagsDo: aBlock
	"Apply the arguments on the direct tag elements of the receiver"
	
	self childrenDo: [ :anElement |
		anElement ifTag: [aBlock value: anElement] ]! !

!SoupTag methodsFor: 'enumerating' stamp: 'sd 3/27/2011 17:33'!
children
	"Returns the direct childent of the receiver so that we can use iterators"
	
	^ children! !

!SoupTag methodsFor: 'enumerating' stamp: 'sd 12/13/2010 21:51'!
childrenDo: aBlock
	"Apply the argument on the direct childent of the receiver"
	
	children do: aBlock! !

!SoupTag methodsFor: 'error handling' stamp: 'sd 12/13/2010 21:44'!
doesNotUnderstand: aMessage
	" Treat only unary messages as short hand attribute or child access."
	(aMessage arguments size > 0)
		ifTrue: [^ super doesNotUnderstand: aMessage].
	attributes 
		at: aMessage selector
		ifPresent: [:aString | ^ aString].
	self 
		findTag: aMessage selector
		ifPresent: [:aTag | ^ aTag].
	^ nil! !

!SoupTag methodsFor: 'finding tags' stamp: 'CamilloBruni 9/21/2011 17:06'!
findAllChildTags: anObject 
	| findBlock |
	findBlock := self createFindTagBlock: anObject.
	^ Array streamContents: [ :aStream |
		self findAllChildTags: findBlock stream: aStream ]! !

!SoupTag methodsFor: 'finding tags' stamp: 'CamilloBruni 9/21/2011 17:04'!
findAllChildTags: anObject stream: aStream
	| findBlock |
	findBlock := self createFindTagBlock: anObject.
	self childTagsDo: [ :aTag |
			(findBlock value: aTag) ifTrue: [aStream nextPut: aTag]].! !

!SoupTag methodsFor: 'finding strings' stamp: 'sd 12/13/2010 22:05'!
findAllStrings: anObject
	"Returns all the strings equal to the argument. Note that the argument can be a string, a Boolean, a regex or a collection of strings to match several elements"
	
	| block |
	block := self createFindStringBlock: anObject.
	^ Array streamContents: [ :aStream |
			self findAllStrings: block stream: aStream ]! !

!SoupTag methodsFor: 'finding tags' stamp: 'sd 12/13/2010 21:41'!
findAllTags: anObject 
	| findBlock |
	findBlock := self createFindTagBlock: anObject.
	^ Array streamContents: [ :aStream |
		self findAllTags: findBlock stream: aStream ]! !

!SoupTag methodsFor: 'finding tags' stamp: 'sd 12/13/2010 21:42'!
findAllTags: anObject stream: aStream
	| findBlock |
	findBlock := self createFindTagBlock: anObject.
	self childTagsDo: [ :aTag |
			(findBlock value: aTag) ifTrue: [aStream nextPut: aTag].
			aTag findAllTags: findBlock stream: aStream ].! !

!SoupTag methodsFor: 'finding tags' stamp: 'sd 12/13/2010 21:42'!
findAllTagsByClass: aString 
	^ Array streamContents:  [ :aStream | 
				self 
					findAllTagsByClass: aString
					stream: aStream ]! !

!SoupTag methodsFor: 'finding tags' stamp: 'CamilloBruni 9/21/2011 15:06'!
findAllTagsByClass: aString stream: aStream 
	self childTagsDo: 
		[ :aTag | 
		(aTag hasClass: aString)
			ifTrue: [ aStream nextPut: aTag ].
		aTag 
			findAllTagsByClass: aString
			stream: aStream ]! !

!SoupTag methodsFor: 'finding strings' stamp: 'sd 12/13/2010 22:08'!
findChildString: anObject
	"Returns the strings matching anObject in the direct children of the receiver. anObject can be a Boolean, a string, a regex or a collection of strings."

	| findBlock |
	findBlock := self createFindStringBlock: anObject.
	self childStringsDo: [ :aString |
		(findBlock value: aString)  ifTrue: [^ aString]].
	^ nil! !

!SoupTag methodsFor: 'finding tags' stamp: 'za 12/23/2008 04:55'!
findChildTag: anObject
	| findBlock |
	findBlock := self createFindTagBlock: anObject.
	self childTagsDo: [
		:aTag |
		(findBlock value: aTag)  ifTrue: [^ aTag].
	].
	^ nil! !

!SoupTag methodsFor: 'finding strings' stamp: 'sd 12/11/2010 16:56'!
findString: anObject
	"Find the first occurrence of a String satisfying the argument. The argument can e a Boolean, a String, a Regex or a collection of strings"
	| block |
	block := self createFindStringBlock: anObject.
	self childrenDo: [ :eachElement |
		eachElement isString
			ifTrue: [(block value: eachElement) ifTrue: [^ eachElement]]
			ifFalse: [
				| found |
				found := eachElement findString: block.
				found ifNotNil: [^ found]
			]
	].
	^ nil
		! !

!SoupTag methodsFor: 'finding strings' stamp: 'sd 12/13/2010 22:10'!
findString: anObject ifPresent: aBlock
	"Execute aBlock, if a string satisfying anObject is found. anObject can be a Boolean, a string, a regex or a collection of strings."
	| found |
	found := self findString: anObject.
	^ found ifNotNil: [aBlock value: found]! !

!SoupTag methodsFor: 'finding tags' stamp: 'za 12/22/2008 22:41'!
findTag: anObject
	| findBlock |
	findBlock := self createFindTagBlock: anObject.
	self childTagsDo: [
		:aTag |
		| found |
		(findBlock value: aTag) 
			ifTrue: [^ aTag].
		found := (aTag findTag: findBlock).
		found ifNotNil: [^ found]
	].
	^ nil! !

!SoupTag methodsFor: 'finding tags' stamp: 'za 12/21/2008 13:56'!
findTag: anObject ifPresent: aBlock
	| found |
	found := self findTag: anObject.
	^ found ifNotNil: [aBlock value: found]
! !

!SoupTag methodsFor: 'finding tags' stamp: 'za 5/14/2009 00:45'!
findTagByClass: aString
	^ self findTagByClass: aString ifAbsent: [nil]
! !

!SoupTag methodsFor: 'finding tags' stamp: 'CamilloBruni 9/21/2011 15:03'!
findTagByClass: aString ifAbsent: aBlock
	self findTag: [:aTag | self hasClass: aString]
		ifPresent: [:aTag | ^ aTag].
	^ aBlock value
! !

!SoupTag methodsFor: 'finding tags' stamp: 'CamilloBruni 9/21/2011 15:03'!
findTagByClass: aString ifPresent: aBlock
	^ self 
		findTag: [:aTag | self hasClass: aString]
		ifPresent: aBlock
! !

!SoupTag methodsFor: 'accessing' stamp: 'sd 3/27/2011 19:15'!
hasAttributes
	^ attributes size > 0 ! !

!SoupTag methodsFor: 'accessing' stamp: 'CamilloBruni 9/21/2011 15:05'!
hasClass: aString
	
	^ (self attributeAt: 'class')
		ifNil: [ false ]
		ifNotNilDo: [ :classAttribute|
			(Character space split: classAttribute) includes: aString ]! !

!SoupTag methodsFor: 'testing' stamp: 'sd 12/13/2010 21:28'!
ifTag: aBlock
	"When the receiver is a tag execute the argument aBlock. The result of the block execution is discarded"
	
	aBlock value! !

!SoupTag methodsFor: 'finding tags' stamp: 'CamilloBruni 9/21/2011 15:20'!
includesTag: aTag
	self childTagsDo: [ :tag|
		tag = aTag
			ifTrue: [ ^ true ].
		(tag includesTag: aTag)
			ifTrue: [ ^ true ]].
	^ false! !

!SoupTag methodsFor: 'initializing' stamp: 'za 12/10/2008 16:21'!
initialize
	super initialize.
	attributes := Dictionary new.
	children := OrderedCollection new.
	isSelfClosing := false.! !

!SoupTag methodsFor: 'testing' stamp: 'za 12/10/2008 16:24'!
isSelfClosing
	^ isSelfClosing! !

!SoupTag methodsFor: 'testing' stamp: 'za 12/9/2008 14:52'!
isTag
	^ true! !

!SoupTag methodsFor: 'accessing' stamp: 'za 12/7/2008 04:24'!
name
	^ name! !

!SoupTag methodsFor: 'accessing' stamp: 'za 12/7/2008 04:24'!
name: anObject
	name := anObject! !

!SoupTag methodsFor: 'printing' stamp: 'sd 12/5/2010 22:49'!
printAttributeValue: aString on: aStream

	| quote string entityMatcher bareSymbolMatcher replacementEntities |
	(aString includes: $")
		ifTrue: 
			[ string := (aString includes: $')
				ifTrue: [aString copyReplaceAll: '''' with: '&squot;']
				ifFalse: [aString].
			quote := $' ]
		ifFalse: 
			[ string := aString.
			quote := $" ].
	bareSymbolMatcher := '[<>&]' asRegex.
	entityMatcher := '(#\d+;)|(#x[0-9a-fA-F]+;)|(\w+;)' asRegex.
	replacementEntities := Dictionary newFrom: {'<' -> '&lt;'. '>' -> '&gt;'. '&' -> '&amp;'}.
	string := bareSymbolMatcher 
						copy: string 
						translatingMatchesUsing: 
							[ :matchedString |
							(entityMatcher matchesStreamPrefix: (string readStream
								position: (bareSymbolMatcher subEnd: 1) first;
								yourself))
								ifTrue: [matchedString]
								ifFalse: [replacementEntities at: matchedString]].
	aStream 
		nextPut: quote;
		nextPutAll: string;
		nextPut: quote.
! !

!SoupTag methodsFor: 'printing' stamp: 'sd 12/5/2010 22:47'!
printOn: aStream
	aStream 
		nextPut: $<;
		nextPutAll: name.
	attributes keysAndValuesDo: 
		[ :key :value |
		aStream 
			nextPut: $ ;
			nextPutAll: key;
			nextPutAll: '='.
		self printAttributeValue: value on: aStream.].
	isSelfClosing 
		ifTrue: [aStream nextPutAll: ' />']
		ifFalse: [
			aStream nextPut: $>.
			children do: 
				[ :eachElement |
				eachElement printOn: aStream].
			aStream 
				nextPutAll: '</';
				nextPutAll: name;
				nextPut: $> ]! !

!SoupTag methodsFor: 'accessing' stamp: 'sd 12/5/2010 22:45'!
size
	"Return the number of direct children an element has."
	
	^ children size! !

!SoupTag methodsFor: 'accessing' stamp: 'sd 12/5/2010 22:37'!
string
	"Return the string representation of the receiver if the receiver is not composed of subnodes."
	^ (children size = 1 and: [(children at: 1) isString])
				ifTrue: [(children at: 1) contents]
				ifFalse: [nil]! !

!SoupTag methodsFor: 'accessing' stamp: 'sd 12/5/2010 22:46'!
text
	"Return a string containing the concatenation of the texts in the receiver. For an example, have a look at (SoupDocumentation>>#accessingText)"

	^ String streamContents: 
			[ :aStream |
				children do: [ :eachElement | 
							aStream nextPutAll: eachElement text ] ]! !

Object subclass: #SoupParser
	instanceVariableNames: 'string currentData tagStack quoteStack interesting startTagOpen charRef entityRef endBracket tagFind attrFind entityOrCharRef previous literal parameters incomplete refFind'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Soup-Core-Parser'!

!SoupParser commentStamp: '<historical>' prior: 0!
SoupParser is a parser that creates Soup trees out of HTML. Soup Elements are not fine grained structure representing detailed HTML elements like a 

Here is an example of how to use it: 

	Soup fromString: 'http://www.bedetheque.com/album-105582-BD-Atomium-Express.html' asUrl retrieveContents contentStream

You can also invoke it using: 
	(SoupParser parameters: Soup defaultParserParameters) 
		parseString:  'http://www.bedetheque.com/album-105582-BD-Atomium-Express.html' asUrl retrieveContents contentStream!

!SoupParser class methodsFor: 'instance creation' stamp: 'za 12/20/2008 11:07'!
parameters: aParameters
	^ self new 
		parameters: aParameters;
		yourself
	! !

!SoupParser methodsFor: 'converting' stamp: 'za 3/9/2009 16:08'!
convertCharRef: aString
	| codePoint |
	[
		codePoint := (aString beginsWith: 'x')
			ifTrue: [Number readFrom: (aString allButFirst: 1) asUppercase base: 16]
			ifFalse: [Integer readFrom: aString readStream].
	] on: Error do: [
		^ nil
	].
	^ self convertCodePoint: codePoint
! !

!SoupParser methodsFor: 'converting' stamp: 'za 12/25/2008 01:54'!
convertCodePoint: codePoint
	^ codePoint asCharacter asString
! !

!SoupParser methodsFor: 'converting' stamp: 'za 12/26/2008 23:11'!
convertEntityRef: aString
	^ parameters entityDefinitions at: aString ifAbsent: [nil]
! !

!SoupParser methodsFor: 'converting' stamp: 'PaulDeBruicker 2/19/2010 10:43'!
convertRef: aMatcher
	(aMatcher subexpression: 3) 
		ifNotNil: [
			^ (self convertCharRef: (aMatcher subexpression: 3))
				ifNil: [aMatcher subexpression: 1]
		].
	(aMatcher subexpression: 4)
		ifNotNil: [
			^ (self convertEntityRef: (aMatcher subexpression: 4))
				ifNil: [aMatcher subexpression: 1]
		].
	^ aMatcher subexpression: 1
! !

!SoupParser methodsFor: 'massaging' stamp: 'za 2/11/2009 04:26'!
copy: aString replace: regexStringOrMatcher using: aBlock
	| matcher |
	matcher := regexStringOrMatcher isMatcher
		ifTrue: [regexStringOrMatcher]
		ifFalse: [regexStringOrMatcher asRegex].
	^ matcher copy: aString translatingMatchesUsing: [
		:matchString |
		" Pass the matcher as it has all the information we need 
		and we can always reconstruct the matched string. "
		aBlock value: matcher.
		]! !

!SoupParser methodsFor: 'initializing' stamp: 'za 12/20/2008 10:56'!
createRootTag
	^ Soup new
! !

!SoupParser methodsFor: 'parsing' stamp: 'za 12/30/2008 04:02'!
decode
	| decoder |
	decoder := SoupDecoder new.
	decoder convertSmartQuotesToHtml.
	decoder string: string.
	string := decoder decode.! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/10/2008 13:05'!
endData
	self endData: SoupString! !

!SoupParser methodsFor: 'handling' stamp: 'za 2/15/2009 18:48'!
endData: aClass
	| data dataWithoutSeparators element |
	(currentData size = 0) ifTrue: [^ self].
	data := currentData contents.
	dataWithoutSeparators := data reject: [:aChar | aChar isSeparator].
	dataWithoutSeparators 
		ifEmpty: [
			data := (data includes: Character cr)
				ifTrue: [String cr]
				ifFalse: [' ']
		].
	currentData := ReadWriteStream on: String new.
	element := aClass parent: tagStack last previous: previous contents: data.
	previous := element.
	 ! !

!SoupParser methodsFor: 'parsing' stamp: 'za 12/10/2008 23:06'!
equals: aString from: anInteger
	(anInteger > string size) ifTrue: [^ false].
	(anInteger + aString size > string size) ifTrue: [^ false].
	aString withIndexDo: [
		:eachChar :eachInteger |
		(string at: (eachInteger + anInteger)) = eachChar
			ifFalse: [^ false]
	].
	^ true! !

!SoupParser methodsFor: 'parsing' stamp: 'PaulDeBruicker 2/19/2010 10:41'!
findInterestingFrom: anInteger
	^ (interesting searchStream: (self readStreamFrom: anInteger))
		ifTrue: [(interesting subBeginning: 1) first]
		ifFalse: [string size]
! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/25/2008 07:02'!
handleCData: aString
	self endData.
	self handleData: aString.
	self endData: SoupCData.

	
! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/24/2008 18:22'!
handleCharRef: aString
	^ self handleData: (String streamContents: [
		:aStream |
		aStream 
			nextPut: $&;
			nextPut: $#;
			nextPutAll: aString;
			nextPut: $;])! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/25/2008 07:10'!
handleComment: aString
	self endData.
	self handleData: aString.
	self endData: SoupComment

	
! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/10/2008 23:52'!
handleData: aString
	currentData nextPutAll: aString

! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/9/2008 16:14'!
handleDataFrom: anInteger to: anotherInteger
	anInteger to: anotherInteger do: [
		:i |
		currentData nextPut: (string at: i)
		]
! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/25/2008 06:41'!
handleDeclaration: aString
	self endData.
	self handleData: aString.
	self endData: SoupDeclaration.

	
! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/23/2008 06:01'!
handleEndTag: aString
	(quoteStack isEmpty not and: [quoteStack last ~= aString])
		ifTrue: [^ self handleData: (String streamContents: [
			:stream | 
				stream nextPutAll: '</'; nextPutAll: aString; nextPutAll: '>'
			])].
	self endData.
	self popToTag: aString.
	(quoteStack isEmpty not and: [quoteStack last = aString])
		ifTrue: [
			quoteStack removeLast.
			literal := quoteStack isEmpty not
		]
! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/23/2008 06:01'!
handleEndTagFrom: anInteger to: anotherInteger tag: aString
	(quoteStack isEmpty not and: [quoteStack last ~= aString])
		ifTrue: [^ self handleDataFrom: anInteger to: anotherInteger].
	self endData.
	self popToTag: aString.
	(quoteStack isEmpty not and: [quoteStack last = aString])
		ifTrue: [
			quoteStack removeLast.
			literal := quoteStack isEmpty not
		]
! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/24/2008 18:21'!
handleEntityRef: aString
	^ self handleData: (String streamContents: [
		:aStream |
		aStream 
			nextPut: $&;
			nextPutAll: aString;
			nextPut: $;])! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/25/2008 07:18'!
handleProcessingInstruction: aString
	self endData.
	self handleData: aString.
	self endData: SoupProcessingInstruction 

	
! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/26/2008 22:56'!
handleStartTagFrom: anInteger to: anotherInteger tag: aString attrs: aDictionary
	| tag |
	quoteStack isEmpty ifFalse: [^ self handleDataFrom: anInteger + 1 to: anotherInteger].
	self endData.
	(self isSelfClosingTag: aString) ifFalse: [self smartPop: aString].
	tag := SoupTag named: aString parent: tagStack last previous: previous.
	aDictionary keysAndValuesDo: [
		:key :value |
		tag attributeAt: key put: value
	].
	previous := tag.
	self pushTag: tag.
	(self isSelfClosingTag: aString) ifTrue: [
		tag beSelfClosing.
		self popTag
	].
	(parameters quoteTags includes: aString) 
		ifTrue: [
			quoteStack addLast: aString.
			literal := true
		].
	^ tag
	

	! !

!SoupParser methodsFor: 'initializing' stamp: 'CamilloBruni 9/21/2011 16:17'!
initialize
	super initialize.
	interesting := '[&<]' asRegex.
	startTagOpen := '<[>a-zA-Z]' asRegex.
	charRef := '&#([0-9]+)[^0-9]' asRegex.
	entityRef := '&([a-zA-Z][-.a-zA-Z0-9]*)[^a-zA-Z0-9]' asRegex.
	endBracket := '[<>]' asRegex.
	tagFind := '[a-zA-Z][-_.a-zA-Z0-9:]*' asRegex.
	attrFind := '\s*([a-zA-Z_][-:.a-zA-Z_0-9]*)(\s*=\s*(''[^'']*''|"[^"]*"|[\-a-zA-Z0-9./,:;+*%?!!&$()_#=~''"@]*))?' asRegex.
	incomplete := '&([a-zA-Z][a-zA-Z0-9]*|#[0-9]*)?|<([a-zA-Z][^<>]*|/([a-zA-Z][^<>]*)?|!![^<>]*)?' asRegex.
	refFind := '&(([a-zA-Z][-.a-zA-Z0-9]*)|#(\w+))(;)?' asRegex.	! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/26/2008 22:37'!
isNestable: aString
	^ parameters nestableTags includesKey: aString
! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/26/2008 22:35'!
isResetNesting: aString
	^ parameters resetNestingTags includesKey: aString! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/20/2008 10:53'!
isSelfClosingTag: aString
	^ parameters selfClosingTags includes: aString! !

!SoupParser methodsFor: 'massaging' stamp: 'za 12/9/2008 15:17'!
massage
	string := self replace: '(<[^<>]*)/>' using: [:aMatcher | (aMatcher subexpression: 2) , '/>'].
	string := self replace: '<!!\s+([^<>]*)>' using: [:aMatcher | '<!!' , (aMatcher subexpression: 2) , '>'].
! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/23/2008 05:42'!
numPops: aString
	(tagStack size to: 2 by: -1) do: [
		:i |
		((tagStack at: i) name = aString) 
			ifTrue: [^ tagStack size - i + 1]
	].
	^ 0! !

!SoupParser methodsFor: 'accessing' stamp: 'za 12/20/2008 11:08'!
parameters: anObject
	parameters := anObject! !

!SoupParser methodsFor: 'parsing' stamp: 'za 12/26/2008 22:33'!
parse
	| position end |
	position := 0.
	end := string size.
	[position < end] whileTrue: [
		position := self parseAny: position.
		position < 0 ifTrue: [^ self]
	]
! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/23/2008 04:25'!
parseAny: anInteger
	| i j |
	i := anInteger.
	j := self findInterestingFrom: i.
	i < j ifTrue: [self handleDataFrom: i + 1 to: j].
	i := j.
	i >= string size ifTrue: [^ string size].
	^ ((string at: i + 1) = $<) 
		ifTrue: [self parsePossibleTag: i]
		ifFalse: [self parsePossibleRef: i]
! !

!SoupParser methodsFor: 'parsing' stamp: 'za 12/26/2008 22:08'!
parseCData: anInteger
	| end |
	end := (string findString: ']]>' startingAt: anInteger) - 1.
	self handleCData: (string copyFrom: anInteger + 10 to: end).
	^ end + 3
! !

!SoupParser methodsFor: 'parsing' stamp: 'PaulDeBruicker 2/19/2010 10:42'!
parseCharRef: anInteger
	self handleCharRef: (charRef subexpression: 2).
	^ ((charRef subEnd: 2) first) + 1
! !

!SoupParser methodsFor: 'parsing' stamp: 'PaulDeBruicker 2/19/2010 10:42'!
parseComment: anInteger
	| matcher |
	matcher := '--\s*>' asRegex.
	(matcher searchStream: (self readStreamFrom: anInteger))
		ifTrue: [
			self handleComment: (string 
				copyFrom: anInteger + 5 
				to: (matcher subBeginning: 1) first).
			^( matcher subEnd: 1) first.
		].
	^ self parseIncomplete: anInteger
		
	
	
! !

!SoupParser methodsFor: 'parsing' stamp: 'PaulDeBruicker 2/19/2010 10:42'!
parseDeclaration: anInteger
	| matcher |
	matcher := '<!!\s*((''[^'']*''|"["]*"|\w+)(\s+(''[^'']*''|"["]*"|\w+)))*>' asRegex.
	(matcher matchesStreamPrefix: (self readStreamFrom: anInteger))
		ifTrue: [
			self handleDeclaration: (matcher subexpression: 2).
			^( matcher subEnd: 1) first.
		].
	^ self parseIncomplete: anInteger
	
! !

!SoupParser methodsFor: 'parsing' stamp: 'PaulDeBruicker 2/19/2010 10:42'!
parseEndTag: anInteger
	| end tag |
	(endBracket searchStream: (self readStreamFrom: anInteger  + 2)) 
		ifFalse: [^ self parseIncomplete: anInteger].
	end := (endBracket subEnd: 1) first.
	tag := string copyFrom: anInteger + 3 to: end - 1. 
	self handleEndTagFrom: anInteger + 1 to: end tag: tag.
	literal := false.
	^ end
! !

!SoupParser methodsFor: 'parsing' stamp: 'PaulDeBruicker 2/19/2010 10:42'!
parseEntityRef: anInteger
	self handleEntityRef: (entityRef subexpression: 2).
	^ ((entityRef subEnd: 2) first) + 1
! !

!SoupParser methodsFor: 'parsing' stamp: 'PaulDeBruicker 2/19/2010 10:43'!
parseIncomplete: anInteger
	| j |
	(incomplete matchesStreamPrefix: (self readStreamFrom: anInteger))
		ifFalse: [
			self handleDataFrom: anInteger to: anInteger.
			^ anInteger + 1
		].
	j := (incomplete subEnd: 1) first.
	j >= string size ifTrue: [^ -1].
	self handleDataFrom: anInteger + 1 to: j.
	^ j
	

	
	
! !

!SoupParser methodsFor: 'parsing' stamp: 'za 12/23/2008 03:52'!
parseLiteral: anInteger
	| i |
	i := anInteger + 1.
	self handleDataFrom: i to: i.
	^ i
	
	
! !

!SoupParser methodsFor: 'parsing' stamp: 'za 12/25/2008 06:12'!
parsePossibleDeclaration: anInteger
	^ (self equals: '<!![CDATA[' from: anInteger)
		ifTrue: [self parseCData: anInteger]
		ifFalse: [self parseDeclaration: anInteger]
	
	
! !

!SoupParser methodsFor: 'parsing' stamp: 'za 12/23/2008 03:37'!
parsePossibleLiteral: anInteger
	^ (string size > (anInteger + 1))
		ifTrue: [self parseLiteral: anInteger]
		ifFalse: [self parseIncomplete: anInteger]! !

!SoupParser methodsFor: 'parsing' stamp: 'za 12/23/2008 04:39'!
parsePossibleOtherTag: anInteger
	^ ((string at: anInteger+2) = $/) 
		ifTrue: [self parseEndTag: anInteger]
		ifFalse: [self parsePossibleSpecialTag: anInteger]
! !

!SoupParser methodsFor: 'parsing' stamp: 'za 12/26/2008 22:22'!
parsePossibleRef: anInteger
	literal ifTrue: [^ self parseLiteral: anInteger].
	(charRef matchesStreamPrefix: (self readStreamFrom: anInteger)) 
		ifTrue: [^ self parseCharRef: anInteger].
	(entityRef matchesStreamPrefix: (self readStreamFrom: anInteger)) 
		ifTrue: [^ self parseEntityRef: anInteger].
	^ self parseIncomplete: anInteger! !

!SoupParser methodsFor: 'parsing' stamp: 'za 12/26/2008 22:23'!
parsePossibleSpecialTag: anInteger
	literal ifTrue: [^ self parsePossibleLiteral: anInteger].
	(self equals: '<!!--' from: anInteger) 
		ifTrue: [^ self parseComment: anInteger].
	((string at: anInteger + 2) = $?)  
		ifTrue: [^ self parseProcessingInstruction: anInteger].
	((string at: anInteger + 2) = $!!)  
		ifTrue: [^ self parsePossibleDeclaration: anInteger].
	^ self parseIncomplete: anInteger
	
	
! !

!SoupParser methodsFor: 'parsing' stamp: 'za 12/23/2008 03:23'!
parsePossibleStartTag: anInteger
	literal ifTrue: [^ self parseLiteral: anInteger].
	^ self parseStartTag: anInteger
	
! !

!SoupParser methodsFor: 'parsing' stamp: 'za 12/23/2008 03:25'!
parsePossibleTag: anInteger
	^ (startTagOpen matchesStreamPrefix: (self readStreamFrom: anInteger))
		ifTrue: [self parsePossibleStartTag: anInteger]
		ifFalse: [self parsePossibleOtherTag: anInteger]
! !

!SoupParser methodsFor: 'parsing' stamp: 'sd 1/20/2011 09:23'!
parseProcessingInstruction: anInteger
	| end |
	end := string findString: '>' startingAt: anInteger +1 .
	(end > 0)
		ifTrue: [
			|  processingInstruction |
			processingInstruction := string 
				copyFrom: anInteger + 3 
				to: end - 1.
			self handleProcessingInstruction: processingInstruction.
			^ end
		].
	^ self parseIncomplete: anInteger
! !

!SoupParser methodsFor: 'parsing' stamp: 'PaulDeBruicker 2/19/2010 10:41'!
parseStartTag: anInteger
	| j k tag attrs |
	(endBracket searchStream: (self readStreamFrom: anInteger + 1))
		ifFalse: [^ self parseIncomplete: anInteger].
	attrs := Dictionary new.
	j := (endBracket subBeginning: 1) first.
	(tagFind matchesStreamPrefix: (self readStreamFrom: anInteger + 1))
		ifFalse: [self error: '#parseStartTagFrom: should not have been sent']
		ifTrue: [
			k := (tagFind subEnd: 1) first.
			tag := (tagFind subexpression: 1) asLowercase
		].
	[k < j] whileTrue: [
		k := self parseStartTagAttribute: k into: attrs.
		(k = -1) ifTrue: [k := j]
	].
	self handleStartTagFrom: anInteger to: j tag: tag attrs: attrs.
	(string at: j + 1) = $> ifTrue: [j := j + 1].
	^ j
! !

!SoupParser methodsFor: 'parsing' stamp: 'PaulDeBruicker 2/19/2010 10:43'!
parseStartTagAttribute: anInteger into: aDictionary
	| attrName rest attrValue |
	(attrFind matchesStreamPrefix: (self readStreamFrom: anInteger))
		ifFalse: [^ -1].
	attrName := attrFind subexpression: 2.
	rest := attrFind subexpression: 3.
	attrValue := attrFind subexpression: 4.
	rest 
		ifNil: [attrValue := attrName]
		ifNotNil: [
			attrValue ifNotEmpty: [
				((attrValue first = $' and: [attrValue last = $']) 
					or: [attrValue first = $" and: [attrValue last = $"]])
					ifTrue: [
						attrValue := attrValue 
							copyFrom: 2 
							to: (attrValue size - 1)
					].
				attrValue := self 
					copy: attrValue 
					replace: refFind
					using: [:aMatcher | self convertRef: aMatcher].
			]
		].
	aDictionary at: attrName put: attrValue.
	^ (attrFind subEnd: 1) first.
! !

!SoupParser methodsFor: 'parsing' stamp: 'sd 1/20/2011 09:23'!
parseString: aString
	string := aString.
	self decode.
	self reset.
	self massage.
	self parse.
	self endData.
	[tagStack last isSoup] whileFalse: [self popTag].
	^ tagStack last! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/26/2008 22:55'!
popTag
	tagStack removeLast.


! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/26/2008 22:51'!
popToTag: aString 
	self popToTag: aString inclusive: true! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/26/2008 22:52'!
popToTag: aString inclusive: aBoolean
	| numPops |
	(aString = Soup defaultName) ifTrue: [^ self].
	numPops := self numPops: aString.
	aBoolean ifFalse: [numPops := numPops - 1].
	numPops timesRepeat: [self popTag].
! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/26/2008 22:55'!
pushTag: aTag
	tagStack addLast: aTag.

! !

!SoupParser methodsFor: 'parsing' stamp: 'za 12/23/2008 03:05'!
readStreamFrom: anInteger
	^ string readStream position: anInteger
! !

!SoupParser methodsFor: 'massaging' stamp: 'za 12/9/2008 15:17'!
replace: regexString using: aBlock
	^ self copy: string replace: regexString using: aBlock
! !

!SoupParser methodsFor: 'initializing' stamp: 'za 12/26/2008 22:55'!
reset
	currentData := ReadWriteStream on: String new.
	tagStack := OrderedCollection new.
	quoteStack := OrderedCollection new.
	literal := false.
	self pushTag: self createRootTag! !

!SoupParser methodsFor: 'handling' stamp: 'za 12/26/2008 22:53'!
smartPop: aString
	tagStack reverseDo: [
		:aTag |
		| nestingResetTriggers |
		((aTag isNil or: [aString = aTag name]) and: [(self isNestable: aString) not])
			ifTrue: [^ self popToTag: aString inclusive: true].
		nestingResetTriggers := parameters nestableTags at: aString ifAbsent: [#()].
		((nestingResetTriggers includes: aTag name) 
			or: [nestingResetTriggers isEmpty 
				and: [(self isResetNesting: aString) 
					and: [self isResetNesting: aTag name]]])
			ifTrue: [^ self popToTag: aTag name inclusive: false]
	]! !

!SoupParser methodsFor: 'accessing' stamp: 'za 12/9/2008 16:11'!
string: anObject
	string := anObject! !

Object subclass: #SoupParserParameters
	instanceVariableNames: 'selfClosingTags nestableInlineTags nestableBlockTags nestableListTags nestableTableTags nestableTags nonNestableBlockTags resetNestingTags quoteTags entityDefinitions'
	classVariableNames: 'ForHtml'
	poolDictionaries: ''
	category: 'Soup-Core-Parser'!

!SoupParserParameters class methodsFor: 'defaults' stamp: 'za 12/20/2008 10:47'!
forHtml
	^ ForHtml ifNil: [
		ForHtml := self basicNew 
			initializeForHtml; 
			yourself
		]! !

!SoupParserParameters class methodsFor: 'defaults' stamp: 'za 12/23/2008 02:51'!
reset
	ForHtml := nil! !

!SoupParserParameters methodsFor: 'accessing' stamp: 'za 12/26/2008 23:12'!
entityDefinitions
	^ entityDefinitions! !

!SoupParserParameters methodsFor: 'accessing' stamp: 'za 12/25/2008 01:22'!
entityDefs
	^ entityDefs! !

!SoupParserParameters methodsFor: 'initializing (html)' stamp: 'za 12/26/2008 23:12'!
initializeEntityDefinitionsForHtml
	entityDefinitions := Dictionary newFrom: {
		'lt' -> '<'.
		'gt' -> '>'.
		'amp' -> '&'.	
		'quot' -> ''''.
		'apos' -> ''''.
	}
! !

!SoupParserParameters methodsFor: 'initializing (html)' stamp: 'za 12/26/2008 23:10'!
initializeForHtml
	self initializeEntityDefinitionsForHtml.
	self initializeSelfClosingTagsForHtml.
	self initializeNestableInlineTagsForHtml.
	self initializeNestableBlockTagsForHtml.
	self initializeNestableListTagsForHtml.
	self initializeNestableTableTagsForHtml.
	self initializeNestableTagsForHtml.
	self initializeNonNestableBlockTagsForHtml.
	self initializeResetNestingTagsForHtml.
	self initializeQuoteTagsForHtml.
! !

!SoupParserParameters methodsFor: 'initializing (html)' stamp: 'za 12/26/2008 23:03'!
initializeNestableBlockTagsForHtml
	nestableBlockTags := Dictionary newFrom: {
		#blockquote -> #().
		#div -> #().
		#fieldset -> #().
		#ins -> #().
		#del -> #()
	}.
! !

!SoupParserParameters methodsFor: 'initializing (html)' stamp: 'za 12/26/2008 23:02'!
initializeNestableInlineTagsForHtml
	nestableInlineTags := Dictionary newFrom: {
		#span -> #().
		#font -> #().
		#q -> #().
		#object -> #().
		#bdo -> #().
		#sub -> #().
		#sup -> #().
		#center -> #()
	}.
! !

!SoupParserParameters methodsFor: 'initializing (html)' stamp: 'za 12/26/2008 23:04'!
initializeNestableListTagsForHtml
	nestableListTags := Dictionary newFrom: {
		#ol -> #().
		#ul -> #().
		#li -> #(ol ul).
		#dl -> #().
		#dd -> #(dl).
		#dt -> #(dl)
	}.
! !

!SoupParserParameters methodsFor: 'initializing (html)' stamp: 'za 12/26/2008 23:05'!
initializeNestableTableTagsForHtml
	nestableTableTags := Dictionary newFrom: {
		#table -> #().
		#tr -> #(table tbody tfoot thead).
		#td -> #(tr).
		#th -> #(td).
		#thead -> #(table).
		#tbody -> #(table).
		#tfoot -> #(table).
	}.
! !

!SoupParserParameters methodsFor: 'initializing (html)' stamp: 'za 12/26/2008 23:06'!
initializeNestableTagsForHtml
	nestableTags := Dictionary new.
	nestableTags
		addAll: nestableInlineTags;
		addAll: nestableBlockTags;
		addAll: nestableListTags;
		addAll: nestableTableTags.
! !

!SoupParserParameters methodsFor: 'initializing (html)' stamp: 'za 12/26/2008 23:07'!
initializeNonNestableBlockTagsForHtml
	nonNestableBlockTags := Dictionary newFrom: {
		#address -> #().
		#form -> #().
		#p -> #().
		#pre -> #().
	}.
! !

!SoupParserParameters methodsFor: 'initializing (html)' stamp: 'za 12/26/2008 23:10'!
initializeQuoteTagsForHtml
	quoteTags := #(script  textarea)
! !

!SoupParserParameters methodsFor: 'initializing (html)' stamp: 'za 12/26/2008 23:08'!
initializeResetNestingTagsForHtml
	resetNestingTags := Dictionary new.
	resetNestingTags 
		addAll: nestableBlockTags;
		addAll: nonNestableBlockTags;
		addAll: nestableListTags;
		addAll: nestableTags;
		at: #noscript put: #().
! !

!SoupParserParameters methodsFor: 'initializing (html)' stamp: 'za 12/26/2008 23:01'!
initializeSelfClosingTagsForHtml
	selfClosingTags := #(br hr input img meta spacer link frame base).
! !

!SoupParserParameters methodsFor: 'accessing' stamp: 'za 12/20/2008 11:10'!
nestableTags
	^ nestableTags! !

!SoupParserParameters methodsFor: 'accessing' stamp: 'za 12/20/2008 11:09'!
quoteTags
	^ quoteTags! !

!SoupParserParameters methodsFor: 'accessing' stamp: 'za 12/20/2008 11:10'!
resetNestingTags
	^ resetNestingTags ! !

!SoupParserParameters methodsFor: 'accessing' stamp: 'za 12/20/2008 11:08'!
selfClosingTags
	^ selfClosingTags! !
